/* example with LCD screen that sends single mesages*/

#include <Arduino.h>
// #include "display.hpp"
#include "display_to_serial.hpp"
#ifdef ESP32
  #include <WiFi.h>
#else
  #include <ESP8266WiFi.h>
#endif
#include <esp_wifi.h>

#define DEBUG_MODE 0

#include "EspNow2MqttClient.hpp"

#include <ArduinoJson.h>
JsonDocument doc;

#define LED_PIN 15

Display display = Display();
#define DISPLAY_LINE_OPERATION 1
#define DISPLAY_LINE_DATA_SENT 2
#define DISPLAY_LINE_DELIVERY_STATUS 3
#define DISPLAY_LINE_RESPONSE 4
#define DISPLAY_LINE__ 5
#define DISPLAY_LINE_ESPNOW_STATUS 6
#define DISPLAY_LINE_MAC 7

#define MESSAGE_TYPE_A 1
#define MESSAGE_TYPE_B 2
const char* queueA = "light";
const char* queueB = "motor";

byte sharedKey[16] = {10,200,23,4,50,3,99,82,39,100,211,112,143,4,15,106}; 
byte sharedChannel = 6 ;
// uint8_t gatewayMac[6] = {0xA4, 0xCF, 0x12, 0x25, 0x9A, 0x30};
uint8_t gatewayMac[6] = {0x54, 0x32, 0x04, 0x0B, 0x2F, 0x64};
EspNow2MqttClient client = EspNow2MqttClient("tstRq", sharedKey, gatewayMac, sharedChannel);

void displayDataOnCompletion( response & rsp)
{
  char line[30];
  int resultCode;

  if  (1 == rsp.opResponses_count)
  {
    resultCode = rsp.opResponses[0].result_code;
    const char * queue = rsp.message_type == MESSAGE_TYPE_A? queueA : queueB ;
    DeserializationError err;
    switch (resultCode)
    {
    case response_Result_OK:
      snprintf(line, sizeof(line), "%s ok: %s", queue, rsp.opResponses[0].payload);
      err = deserializeJson(doc, (const byte*)rsp.opResponses[0].payload);
      if (!err) { //检查反序列化是否成功
        if(doc.containsKey("status")) {
          int val = doc["status"];
          if(val) {
            Serial.println("Set LED On");
            digitalWrite(LED_PIN, HIGH);
          } else {
            Serial.println("Set LED Off");
            digitalWrite(LED_PIN, LOW);
          }
        }
      } else {
        Serial.print(F("deserializeJson() failed: "));
        Serial.println(err.c_str());
      }
      break;
    case response_Result_NO_MSG:
#if DEBUG_MODE
      snprintf(line, sizeof(line), "%s ok but no msg %d", queue, rsp.message_type);
#endif
      break;
    default:
#if DEBUG_MODE
      snprintf(line, sizeof(line), "%s status: %d %s", queue, resultCode, rsp.opResponses[0].payload);
#endif
      break;
    }
  } else {
    snprintf(line, sizeof(line), "error: %d", rsp.opResponses_count);
  }
#if DEBUG_MODE
  display.print(DISPLAY_LINE_RESPONSE,line,true);
#else
  if (resultCode == response_Result_OK){
    display.print(DISPLAY_LINE_RESPONSE,line,true);
  }
#endif
  // Serial.println(line);
}

void onDataSentUpdateDisplay(bool success) {
#if DEBUG_MODE
  display.print(DISPLAY_LINE_DELIVERY_STATUS, success ? "Delivery Success" : "Delivery Fail", false);
#endif
}

void displayGwMac()
{
  char macStr[24];
  snprintf(macStr, sizeof(macStr), "GwMac %02x:%02x:%02x:%02x:%02x:%02x",
         gatewayMac[0], gatewayMac[1], gatewayMac[2], gatewayMac[3], gatewayMac[4], gatewayMac[5]);
  display.print(DISPLAY_LINE_MAC, macStr);
}

void displayMyMac()
{
  char macStr[22];
  strcpy(macStr, "Mac ");
  strcat(macStr,WiFi.macAddress().c_str());
  display.print(DISPLAY_LINE_MAC, macStr);
}

int32_t getWiFiChannel(const char *ssid) {
  if (int32_t n = WiFi.scanNetworks()) {
      for (uint8_t i=0; i<n; i++) {
          if (!strcmp(ssid, WiFi.SSID(i).c_str())) {
              return WiFi.channel(i);
          }
      }
  }
  return 0;
}

void testRequest(int msgType)
{
  bool ret;
  const char* queue = MESSAGE_TYPE_A == msgType ? queueA: queueB;
#if DEBUG_MODE
  display.print(DISPLAY_LINE_OPERATION, "request", false);
  display.print(DISPLAY_LINE_DATA_SENT, queue, false);
#endif
  ret = client.doSubscribe(queue, msgType);
#if DEBUG_MODE
  if(ret) {
    display.print(DISPLAY_LINE_MAC, "doSubscribe OK.");
  } else {
    display.print(DISPLAY_LINE_MAC, "doSubscribe Fail.");
  }
  display.print(DISPLAY_LINE_MAC, "\n");
#endif
}

void setup() {
  Serial.begin(115200);
  delay(1000);
  pinMode(LED_PIN, OUTPUT);

  display.init();
  displayGwMac();
  displayMyMac();

  int initcode;
  do {
    display.print(DISPLAY_LINE_ESPNOW_STATUS, "             TRYING");
    initcode = client.init();
    switch (initcode)
    {
    case 1:
      display.print(DISPLAY_LINE_ESPNOW_STATUS ,"CANNOT INIT");
      break;
    case 2:
      display.print(DISPLAY_LINE_ESPNOW_STATUS ,"CANNOT PAIR");
      break;
    default:
      display.print(DISPLAY_LINE_ESPNOW_STATUS ,"PAIRED");
      break;
    }
    delay(1001);
  } while (initcode != 0);

  client.onSentACK = onDataSentUpdateDisplay;
  client.onReceiveSomething = displayDataOnCompletion;
}

void loop() {
    testRequest (MESSAGE_TYPE_A);
    // delay(3000);
    // testRequest (MESSAGE_TYPE_B);
    delay(1000);
}